# process coverage bed files

# define




bedfile_long <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata2/pipeline_results/13-SA01718/unicycler/stats/coverage_longreads.bed"

bedfile_short <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata2/pipeline_results/13-SA01718/unicycler/stats/coverage_shortreads.bed"

bedcov_short <- read.delim(bedfile_short, sep = "\t", stringsAsFactors = F, header = F, col.names = c("REF","POS","DEPTH"))
bedcov_long <- read.delim(bedfile_long, sep = "\t", stringsAsFactors = F, header = F, col.names = c("REF","POS","DEPTH"))  

# check 
if (nrow(bedcov) == 0) stop (bedfile, " contains no data")

# average per contig


union(unique(bedcov_short$REF),unique(bedcov_long$REF))

contigs_short <- unique(bedcov_short$REF)



Get_Coverage_Stats <- function(bedcov){
  
  contigs <- unique(bedcov$REF)
  
  contig_coverage_stats <- do.call(rbind,lapply(contigs, function(x){
    bedcov_contig <- bedcov %>% filter(REF == x)
    
    contig_mean <- mean(bedcov_contig$DEPTH)
    bases_uncovered <- sum(bedcov_contig$DEPTH == 0)
    contig_length <- length(bedcov_contig$DEPTH)
    uncoverage <- bases_uncovered/contig_length
    coverage <- 1 - uncoverage
    
    data.frame(contig=x, length= contig_length, mean_depth = contig_mean, bases_uncovered = bases_uncovered, uncoverage = uncoverage, coverage = coverage)
    
    
  }))
  
  return(contig_coverage_stats)

}


bedstats_long <- Get_Coverage_Stats(bedcov_long)
bedstats_short <- Get_Coverage_Stats(bedcov_short)

colnames(bedstats_long)[-c(1)] <- paste0(colnames(bedstats_long)[-c(1)],"_longreads")
colnames(bedstats_short)[-c(1)] <- paste0(colnames(bedstats_short)[-c(1)],"_shortreads")

merge(bedstats_long,bedstats_short,by="contig")


# bin data ----


binnumber <- 100

bedcov <- bedcov_short

bedcov$bins <- cut(bedcov$POS, breaks = binnumber)


# split per reference
refs = unique(bedcov$REF)
bedcov_split <- lapply(refs, function(ref) bedcov[bedcov$REF == ref,])


# plot

# remove uncovered plasmids
bedcov_split <- bedcov_split[sapply(bedcov_split, function(x) sum(as.numeric(x$DEPTH)) > 0)]

# plot all references

library(dplyr)
library(plyr)

plot.list <- lapply(bedcov_split, function(mybedcov){
  chrom = mybedcov$REF[1]
  mybedcov_binned <- ddply(mybedcov, "bins", summarize, meanDepth = sum(as.numeric(DEPTH)), meanPos = (min(POS) + max(POS))/2 )
  mybedcov_binned$normalizedmeanDepth <- mybedcov_binned$meanDepth/sum(mybedcov_binned$meanDepth)
  
  p1 <- ggplot(mybedcov_binned, aes(x=meanPos, y=normalizedmeanDepth)) + 
    geom_bar(stat="identity") + 
    labs(title=paste("Coverage map for sample",sample,", chromosome:",chrom), y = "") + 
    #    theme(axis.text.x = element_blank(), axis.text.y = element_blank(), axis.ticks = element_blank(),plot.margin = unit(c(0,0,0,0), "cm")) + 
    theme(axis.text.x = element_text(angle=45)) + #,plot.margin = unit(c(0,0,0,0), "cm")) + 
    scale_x_discrete(name = "", limit = seq(0,max(mybedcov_binned$meanPos), by = 10^5))#,labels = scales::scientific)
  
  return(p1)
})

args.list <- c(plot.list, list(ncol = 1, bottom = "Genome position", left = "Depth (normalized to bin)"))


do.call(grid.arrange, args.list)



plot.list[[1]]
plot.list[[2]]
plot.list[[3]]
plot.list[[4]]
