# create flye stats in analogy to unicycler stats


suppressPackageStartupMessages(library(Biostrings, quietly = T))


# file definitions

# manual:
# input
#assembly_in.file <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata/pipeline_results/18-SA02383-0/flye/pilon/assembly.fasta"
# stats_in.file <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata/pipeline_results/18-SA02383-0/flye/assembly_info.txt"
#output
# assembly_out.file <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata/pipeline_results/18-SA02383-0/flye/18-SA02383-0.fasta"
# stats_out.file <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata/pipeline_results/18-SA02383-0/flye/18-SA02383-0_stats.tsv"

# input
assembly_in.file <- snakemake@input[["flye_assembly"]]
stats_in.file <- snakemake@input[["stats"]]
# output
assembly_out.file <- snakemake@output[["final_assembly"]]
stats_out.file <- snakemake@output[["flye_summary"]]


# load data

assembly_in <- Biostrings::readDNAStringSet(assembly_in.file)
stats_in <- read.delim(stats_in.file, stringsAsFactors = F)

# remove pilon entries from header
assembly_names.corrected <- gsub("_pilon","",names(assembly_in))
assembly_in_stats <- data.frame(assembly_names.corrected, length.polished=width(assembly_in))

# merge data
stats_raw <- merge(stats_in,assembly_in_stats,by.x = "X.seq_name", by.y= "assembly_names.corrected")

if(nrow(stats_raw) != nrow(assembly_in_stats)){
  stop("Error: contig number and assembly stats have different length")
}

# format data
stats_raw$circular <- ifelse(stats_raw$circ. == "Y","circular","linear")

stats_out <- stats_raw[,c("X.seq_name","length.polished","cov.","circular")]
colnames(stats_out) <- c("contig","length","depth","circularity")


# fix assembly header
new_header <- apply(stats_out,1,function(x){
  paste0(x["contig"]," ","length=",x["length"]," ", "depth=",x["depth"]," ",ifelse(x["circularity"]=="circular","circular=true",""))
})

new_header <- sub("=[ ]+","=",new_header)

# rename assembly
names(assembly_in) <- new_header

# write out

write.table(stats_out, stats_out.file, sep = "\t", row.names = F, quote = F)

writeXStringSet(assembly_in, assembly_out.file)
