# TODO classify as plasmid or not

# load platon data
platon.file <- "/cephfs/abteilung4/Projects_NGS/FARMED/Ringtrial_analysis/analysis/test_data/13-AB01897_Ecoli_mc5/13-AB01897_Ecoli_mc5/flye/platon/13-AB01897_Ecoli_mc5.tsv"

platon <- read.delim(platon.file, stringsAsFactors = F, check.names = T, colClasses = c("character",rep("numeric",4),"character",rep("numeric",8)))


classify_platon <- function(platon, length_threshold = 1000000,RDS_sensitivity_threshold = -7.7, RDS_specificity_threshold = 0.4,RDS_conservative_threshold = 0.1, check=T){

  # check required colnames
  if(check){
    expected_colnames <- c("ID","Length","Coverage","X..ORFs","RDS","Circular","Inc.Type.s.","X..Replication","X..Mobilization","X..OriT","X..Conjugation","X..AMRs","X..rRNAs","X..Plasmid.Hits")  
    if(!all(colnames(platon) == expected_colnames)){
      stop("Unexpected column names:\nExpected:",expected_colnames,"\nProvided:",colnames(platon))
    }
  }

  platon_class <- apply(platon,1,function(mycontig){
    if(mycontig["Length"] > length_threshold) {
      return("chromosome")
    } else if (as.numeric(mycontig["RDS"]) < RDS_sensitivity_threshold) {
      return("chromosome")
    } else if (as.numeric(mycontig["RDS"]) > RDS_specificity_threshold) {    
      return("plasmid")    
    } else if (mycontig["Circular"] == "yes") {    
      return("plasmid")
    } else if (as.numeric(mycontig["Inc.Type.s."]) > 0) {    
      return("plasmid")    
    } else if (as.numeric(mycontig["X..Replication"]) > 0) {    
      return("plasmid")    
    } else if (as.numeric(mycontig["X..OriT"]) > 0) {    
      return("plasmid")
    } else if (as.numeric(mycontig["RDS"]) > RDS_conservative_threshold & as.numeric(mycontig["X..Plasmid.Hits"]) > 0  & as.numeric(mycontig["X..rRNAs"]) == 0) {
      return("plasmid")    
    } else {
      return("unknown")
    }
  })
  
  platon$Classification <- platon_class
  
  return(platon)
  
}
  
platon_2 <- classify_platon(platon)

table(platon_2$Classification)

