# find genes on contig tips

suppressPackageStartupMessages(library(Biostrings, quietly = T))
suppressPackageStartupMessages(library(dplyr, quietly = T))

# input
# abricate result

# contig stats: start end, or length

abricate.file <- snakemake@input[["abricate_report"]]
assembly.file <- snakemake@input[["final_assembly"]]
outfile <- snakemake@output[["summary"]]

# redirect log
log <- file(snakemake@log[[1]], open="wt")
sink(log)
sink(log, type="message")




# manual

#abricate.file <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata/pipeline_results/18-SA02383-0/flye/abricate/report_ncbi_mincov20.tsv"

#abricate.file <- "/cephfs/abteilung4/borowiak/13-SA01718_Stammvergleich/duplication_check/abricate_ncbi.tsv"
#assembly.file <- "/home/DenekeC/Snakefiles/long_read_assembly_pipeline/testdata/pipeline_results/18-SA02383-0/flye/18-SA02383-0.fasta"
#outfile <- "/cephfs/abteilung4/borowiak/13-SA01718_Stammvergleich/duplication_check/contig_end_genes.tsv"





# read in data

abricate <- read.delim(abricate.file, stringsAsFactors = F)

#flye_info.file <- "/cephfs/abteilung4/Datentransfer/MinIon_Network/180921_albacore_basecalling/13-SA01718/flye/assembly_info.txt"
#flye_info <- read.delim(flye_info.file, stringsAsFactors = F)
#flye_info$contigname2 <- paste0(flye_info$X.seq_name,"_pilon")


#flye_info.file <- "/cephfs/abteilung4/borowiak/13-SA01718_Stammvergleich/duplication_check/MinIon_flye.length.tsv"
#flye_info <- read.delim(flye_info.file, stringsAsFactors = F, header = F, col.names = c("contig","length"))


assembly <- Biostrings::readDNAStringSet(assembly.file)
flye_info <- data.frame(contig=names(assembly),length=width(assembly))


# 



contig_set <- unique(flye_info$contig)

#foreach (contig = contig_set)%do%{
overlapping_genes <- lapply(contig_set, function(contig){
  print(paste("Processing contig",contig))
  
  contig_length <- flye_info$length[flye_info$contig == contig]
  print(paste("length:",contig_length))
  
  
  
  abricate_contig <- abricate %>% filter(SEQUENCE == contig)
  
  
  if(nrow(abricate_contig) > 0){
    latest_gene_end <- tail(abricate_contig$END,1)  
    print(paste("latest_gene_end:",latest_gene_end))
    if(latest_gene_end == contig_length)  {
      print("POSSIBLE CONTIG END MATCH")
      abricate_tips <- abricate_contig[c(1,nrow(abricate_contig)),]
      print(abricate_tips)
      
      if (abricate_tips[1,"GENE"] == abricate_tips[2,"GENE"]){
        print(paste("Same gene:",abricate_tips[1,"GENE"]))
        
        joined_coverage <- abricate_tips[1,"X.COVERAGE"] + abricate_tips[2,"X.COVERAGE"]
        print(paste("joined_coverage:",joined_coverage))
        
        df <- data.frame(contig=contig,gene=abricate_tips[1,"GENE"],coverage_start=abricate_tips[1,"X.COVERAGE"],coverage_end=abricate_tips[2,"X.COVERAGE"],coverage_sum=joined_coverage,coverage_coordinates_start=abricate_tips[1,"COVERAGE"],coverage_coordinates_end=abricate_tips[2,"COVERAGE"])
        return(df)
      }

    }
    
  } else {
    print("No gene found")
  }
  
  df <- data.frame(contig=contig,gene=NA,coverage_start=NA,coverage_end=NA,coverage_sum=NA,coverage_coordinates_start=NA,coverage_coordinates_end=NA)
  return(df)
} # end for all contigs
)


overlapping_genes <- do.call(rbind,overlapping_genes)

if(nrow(overlapping_genes) > 1){
  overlapping_genes <- overlapping_genes[!is.na(overlapping_genes$gene),]  
}



# save
write.table(overlapping_genes,outfile, sep="\t", quote = F, row.names = F)
